/*
 * buttons.c
 *
 * Created: 19.04.2017 12:05:09
 *  Author: dsosnowski
 */ 
#define F_CPU 8000000
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include "buttons.h"

#define BUTT_PORT			PORTD
#define BUTT_DDR			DDRD
#define BUTT_IN				PIND
#define BUTT1_PIN			2
#define BUTT2_PIN			1
#define BUTT3_PIN			0
#define NUM_OF_BUTTONS		3

#define BUTTON_CLICK		2
#define BUTTON_THRESHOLD	64
#define BUTTON_REPEAT_CYCLE	10


struct {
	uint8_t action;
}butt;

//----------------------------------------------------------------------
void ButtonInit(void){
	BUTT_DDR  &= ~(1<<BUTT1_PIN | 1<<BUTT2_PIN | 1<<BUTT3_PIN);
	BUTT_PORT |= (1<<BUTT1_PIN | 1<<BUTT2_PIN | 1<<BUTT3_PIN);
}

//----------------------------------------------------------------------
uint8_t ButtonProc(void){
	uint8_t button = 0;
	static uint8_t prev = 0;
	static uint8_t hold = 0;
		
	if ((BUTT_IN & (1<<BUTT1_PIN)) == 0) button = BUTTON1_CLICK;
	if ((BUTT_IN & (1<<BUTT2_PIN)) == 0) button = BUTTON2_CLICK;
	if ((BUTT_IN & (1<<BUTT3_PIN)) == 0) button = BUTTON3_CLICK;

	//no button
	if (button == 0) {
		if (prev > 0) {
			if ((hold >= BUTTON_CLICK) && (hold < BUTTON_THRESHOLD))
			button = prev;
		}
		prev = 0;			//sw_free
		hold = 0;
	
	//any button
	} else {
		if (hold < 0xFF) hold++;		//counting hold time
		if (button != prev) hold = 0;	//new key - reset counters

		prev = button;
		button = 0;
		
		if (hold == BUTTON_THRESHOLD) {
			button = (prev | BUTTON_HOLD);
		}
		
		if (hold > (BUTTON_THRESHOLD + BUTTON_REPEAT_CYCLE)) {
			if ((prev == BUTTON2_CLICK) || (prev == BUTTON3_CLICK)){
				//button = (prev | BUTTON_REPEAT);
				button = (prev);
				hold -= BUTTON_REPEAT_CYCLE;
			}
		}
	}
	butt.action = button;
	return button;	
}

//----------------------------------------------------------------------
uint8_t ButtonActionGet(void){
	return butt.action;
}